<?php

declare(strict_types=1);

namespace apexl\Io\modules\user\Request;

use apexl\Io\Request\IoRequest;

final class LoginByKeyRequest extends IoRequest
{
    public function nfcKey(): string
    {
        return $this->bodyParam('nfc_key');
    }

    public function redirect(): ?string
    {
        return $this->bodyParam('redirect');
    }

    public function validate(): void
    {
//        $this->requireBody('nfc_key');
    }
}